"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const b_node_1 = require("./b-node");
describe('hasLeft', function () {
    it('should be true', function () {
        let root = new b_node_1.BNode(1);
        let left = new b_node_1.BNode(2);
        root.left = left;
        expect(root.hasLeft()).toBe(true);
    });
    it('should be false', function () {
        let root = new b_node_1.BNode(1);
        expect(root.hasLeft()).toBe(false);
        root.left = null;
        expect(root.hasLeft()).toBe(false);
    });
});
describe('hasRight', function () {
    it('should be true', function () {
        let root = new b_node_1.BNode(1);
        let right = new b_node_1.BNode(2);
        root.right = right;
        expect(root.hasRight()).toBe(true);
    });
    it('should be false', function () {
        let root = new b_node_1.BNode(1);
        expect(root.hasLeft()).toBe(false);
        root.left = null;
        expect(root.hasLeft()).toBe(false);
    });
});
describe('data', function () {
    it('should return 1', function () {
        let node = new b_node_1.BNode(1);
        expect(node.data).toEqual(1);
    });
    it('should set 2', function () {
        let node = new b_node_1.BNode(1);
        node.data = 2;
        expect(node.data).toEqual(2);
    });
});
describe('left', function () {
    it('should return 1', function () {
        let root = new b_node_1.BNode(1);
        let left = new b_node_1.BNode(2);
        root.left = left;
        expect(root.left.data).toEqual(2);
    });
    it('should set 2', function () {
        let root = new b_node_1.BNode(1);
        let left = new b_node_1.BNode(2);
        root.left = left;
        expect(root.left.data).toEqual(2);
    });
});
describe('right', function () {
    it('should return 1', function () {
        let root = new b_node_1.BNode(1);
        let right = new b_node_1.BNode(2);
        root.right = right;
        expect(root.right.data).toEqual(2);
    });
    it('should set 2', function () {
        let root = new b_node_1.BNode(1);
        let right = new b_node_1.BNode(2);
        root.right = right;
        expect(root.right.data).toEqual(2);
    });
});
describe('add', function () {
    it('add undefined', function () {
        let root = new b_node_1.BNode(undefined);
        expect(root.add(2)).toBe(false);
    });
    it('add left', function () {
        let root = new b_node_1.BNode(5);
        expect(root.add(2)).toBe(true);
    });
    it('add right', function () {
        let root = new b_node_1.BNode(5);
        expect(root.add(6)).toBe(true);
    });
    it('add two node right subtree', function () {
        let root = new b_node_1.BNode(5);
        root.add(6);
        root.add(7);
        expect(root.right.data).toEqual(6);
        expect(root.right.right.data).toEqual(7);
    });
    it('add two node left subtree', function () {
        let root = new b_node_1.BNode(5);
        root.add(4);
        root.add(3);
        expect(root.left.data).toEqual(4);
        expect(root.left.left.data).toEqual(3);
    });
    it('add left then rigth', function () {
        let root = new b_node_1.BNode(5);
        root.add(4);
        root.add(6);
        expect(root.left.data).toEqual(4);
        expect(root.right.data).toEqual(6);
    });
});
describe('return min value', function () {
    it('should return same node value', function () {
        let node1 = new b_node_1.BNode(5);
        let node2 = new b_node_1.BNode(6);
        expect(node1.getMinValue()).toEqual(5);
    });
    it('should return min value', function () {
        let node1 = new b_node_1.BNode(5);
        let node2 = new b_node_1.BNode(6);
        let node3 = new b_node_1.BNode(4);
        node1.right = node2;
        node1.left = node3;
        expect(node1.getMinValue()).toEqual(4);
    });
});
describe('remove', function () {
    it('should return left undefined', function () {
        let node2 = new b_node_1.BNode(4);
        let node3 = new b_node_1.BNode(5);
        node2.right = node3;
        node2.left = undefined;
        expect(node2.remove(node2, 2)).toBe(false);
    });
    it('should return right undefined', function () {
        let node2 = new b_node_1.BNode(5);
        let node3 = new b_node_1.BNode(4);
        node2.left = node3;
        node2.right = undefined;
        expect(node2.remove(node2, 6)).toBe(false);
    });
    it('should remove left node', function () {
        let node2 = new b_node_1.BNode(5);
        let node3 = new b_node_1.BNode(4);
        let node4 = new b_node_1.BNode(3);
        node2.left = node3;
        node3.left = node4;
        expect(node2.remove(node2, 4)).toBe(true);
    });
    it('should remove right node', function () {
        let node2 = new b_node_1.BNode(5);
        let node3 = new b_node_1.BNode(6);
        let node4 = new b_node_1.BNode(7);
        node2.right = node3;
        node3.right = node4;
        expect(node2.remove(node2, 6)).toBe(true);
    });
    it('should remove min value', function () {
        let node2 = new b_node_1.BNode(5);
        let node3 = new b_node_1.BNode(6);
        let node4 = new b_node_1.BNode(4);
        let node5 = new b_node_1.BNode(3);
        let node6 = new b_node_1.BNode(5);
        node2.right = node3;
        node2.left = node4;
        node4.left = node5;
        node4.right = node6;
        expect(node2.remove(node2, 4)).toBe(true);
    });
});
//# sourceMappingURL=b-node.spec.js.map